import type { WithElement } from './types';
/**
 * Base class for {@link PageObject} that contains stub implementations of a
 * subset of array methods (the ones that produce a possibly useful value). The
 * actual functionality is handled by the proxy, the implementations aren't
 * important, these are just here for type checking.
 *
 * It really seems like there should be a better way to do this, but I can't
 * figure out how else to make PageObject subclasses look like arrays of
 * `WithElement<this, ElementType>`.
 *
 * @private
 */
export default class ArrayStub<ElementType extends Element> {
    /**
     * @private
     */
    readonly length: number;
    /**
     * @private
     */
    pop: () => WithElement<this, ElementType> | undefined;
    /**
     * @private
     */
    reverse: () => WithElement<this, ElementType>[];
    /**
     * @private
     */
    shift: () => WithElement<this, ElementType> | undefined;
    /**
     * @private
     */
    slice: (_start?: number, _end?: number) => WithElement<this, ElementType>[];
    /**
     * @private
     */
    sort: (_compareFn?: (a: WithElement<this, ElementType>, b: WithElement<this, ElementType>) => number) => this[];
    /**
     * @private
     */
    every: (predicate: (value: WithElement<this, ElementType>, index: number, array: WithElement<this, ElementType>[]) => unknown, _thisArg?: any) => boolean;
    /**
     * @private
     */
    some: (_predicate: (value: WithElement<this, ElementType>, index: number, array: WithElement<this, ElementType>[]) => unknown, _thisArg?: any) => boolean;
    /**
     * @private
     */
    forEach: (_callbackfn: (value: WithElement<this, ElementType>, index: number, array: WithElement<this, ElementType>[]) => void, _thisArg?: any) => void;
    /**
     * @private
     */
    map: <U>(_callbackfn: (value: WithElement<this, ElementType>, index: number, array: WithElement<this, ElementType>[]) => U, _thisArg?: any) => U[];
    /**
     * @private
     */
    filter: (predicate: (value: WithElement<this, ElementType>, index: number, array: WithElement<this, ElementType>[]) => unknown, thisArg?: any) => WithElement<this, ElementType>[];
    /**
     * @private
     */
    reduce: <U>(_callbackfn: (previousValue: U, currentValue: WithElement<this, ElementType>, currentIndex: number, array: WithElement<this, ElementType>[]) => U, initialValue: U) => U;
    /**
     * @private
     */
    reduceRight: <U>(_callbackfn: (previousValue: U, currentValue: WithElement<this, ElementType>, currentIndex: number, array: WithElement<this, ElementType>[]) => U, initialValue: U) => U;
    /**
     * @private
     */
    find: (predicate: (value: WithElement<this, ElementType>, index: number, obj: WithElement<this, ElementType>[]) => unknown, thisArg?: any) => WithElement<this, ElementType> | undefined;
    /**
     * @private
     */
    findIndex: (_predicate: (value: WithElement<this, ElementType>, index: number, obj: WithElement<this, ElementType>[]) => unknown, _thisArg?: any) => number;
    /**
     * @private
     */
    [Symbol.iterator]: () => IterableIterator<WithElement<this, ElementType>>;
    /**
     * @private
     */
    entries: () => IterableIterator<[
        number,
        WithElement<this, ElementType>
    ]>;
    /**
     * @private
     */
    keys: () => IterableIterator<number>;
    /**
     * @private
     */
    values: () => IterableIterator<WithElement<this, ElementType>>;
    /**
     * @private
     */
    readonly [n: number]: this;
}
