import PageObject from './page-object';
import type { PageObjectConstructor } from './-private/types';
/**
 * Define a child PageObject. It can optionally be supplied with a
 * {@link PageObject} subclass definition to allow customizing functionality on
 * the page object, such as defining (grand)child page objects, or other helpful
 * properties and functions. Alternatively, it can be passed an {@link Element}
 * subclass as a type argument so elements its page objects produces will be
 * typed more specifically.
 *
 * @param {string} selector the selector relative to the parent node
 *
 * @returns {PageObject} a {@link PageObject} instance
 */
export default function selector<ElementType extends Element = Element>(selector: string): PageObject<ElementType>;
/**
 * Define a child PageObject. It can optionally be supplied with a
 * {@link PageObject} subclass definition to allow customizing functionality on
 * the page object, such as defining (grand)child page objects, or other helpful
 * properties and functions. Alternatively, it can be passed an {@link Element}
 * subclass as a type argument so elements its page objects produces will be
 * typed more specifically.
 *
 * @param {string} selector the selector relative to the parent node
 * @param {Function<PageObject>} [Class] {@link PageObject} subclass that
 * can be used to extend the functionality of this page object
 *
 * @returns {PageObject} a {@link PageObject} subclass instance
 */
export default function selector<ElementType extends Element, T extends PageObject<ElementType>>(selector: string, Class: PageObjectConstructor<ElementType, T>): T;
